@@ -11,7 +11,8 @@
 /*	TLS_RPT *trw_create(
 /*	const char *rpt_socket_name,
 /*	const char *rpt_policy_domain,
-/*	const char *rpt_policy_string)
+/*	const char *rpt_policy_string,
+/*	int	skip_reused_hs)
 /*
 /*	void	trw_free(
 /*	TLSRPT_WRAPPER *trw)
@@ -45,6 +46,9 @@
 /*	int	trw_is_reported(
 /*	TLSRPT_WRAPPER *trw)
 /*
+/*	int	trw_is_skip_reused_hs(
+/*	TLSRPT_WRAPPER *trw)
+/*
 /*	tlsrpt_policy_type_t convert_tlsrpt_policy_type(
 /*	const char *policy_type)
 /*
@@ -136,6 +140,10 @@
 /* .IP rpt_policy_string
 /*	The TLSRPT policy record content, i.e. how to submit TLSRPT
 /*	summary reports.
+/* .IP skip_reused_hs
+/*	If non-zero, do not report the TLSRPT status for TLS handshakes
+/*	that reuse a previously-negotiated TLS session; such sessions
+/*	were already reported when they were used for the first time.
 /* .PP
 /*	trw_free() destroys storage allocated with other trw_xxx()
 /*	requests.
@@ -199,6 +207,9 @@
 /*	trw_is_reported() returns non-zero when the contents of the
 /*	specified TLSRPT_WRAPPER have been reported.
 /* .PP
+/*	trw_is_skip_reused_hs() returns non-zero if the skip_reused_hs
+/*	argument of trw_create() was non-zero.
+/* .PP
 /*	convert_tlsrpt_policy_type() and convert_tlsrpt_policy_failure()
 /*	convert a valid policy type or failure name to the corresponding
 /*	enum value. The result is < 0 if the name is not valid.
@@ -298,10 +309,18 @@
 
 TLSRPT_WRAPPER *trw_create(const char *rpt_socket_name,
 			           const char *rpt_policy_domain,
-			           const char *rpt_policy_string)
+			           const char *rpt_policy_string,
+			           int skip_reused_hs)
 {
+    const char myname[] = "trw_create";
     TLSRPT_WRAPPER *trw;
 
+    if (msg_verbose > 1)
+	msg_info("%s(rpt_socket_name=%s, rpt_policy_domain=%s, "
+		 "rpt_policy_string=%s, skip_reused_hs=%d)",
+		 myname, rpt_socket_name, rpt_policy_domain,
+		 rpt_policy_string, skip_reused_hs);
+
     /*
      * memset() is not portable for pointer etc. types.
      */
@@ -317,6 +336,7 @@ TLSRPT_WRAPPER *trw_create(const char *rpt_socket_name,
     trw->rcv_mta_name = 0;
     trw->rcv_mta_addr = 0;
     trw->rcv_mta_ehlo = 0;
+    trw->skip_reused_hs = skip_reused_hs;
     trw->flags = 0;
     return (trw);
 }
@@ -325,6 +345,10 @@ TLSRPT_WRAPPER *trw_create(const char *rpt_socket_name,
 
 void    trw_free(TLSRPT_WRAPPER *trw)
 {
+    if (msg_verbose > 1)
+	msg_info("trw_free: rpt_socket_name=%s, rpt_policy_domain=%s, ...",
+		 trw->rpt_socket_name, trw->rpt_policy_domain);
+
     /* Destroy fields set with trw_create(). */
     myfree(trw->rpt_socket_name);
     myfree(trw->rpt_policy_domain);
@@ -349,6 +373,19 @@ void    trw_set_tls_policy(TLSRPT_WRAPPER *trw,
 			           const char *tls_policy_domain,
 			           const char *const * mx_host_patterns)
 {
+    const char myname[] = "trw_set_tls_policy";
+
+#define STR_OR_NULL(s)	((s) ? (s) : "(Null)")
+#define PSTR_OR_NULL(p)	((p) ? STR_OR_NULL(*p) : "(Null)")
+
+    if (msg_verbose > 1)
+	msg_info("%s(tlsrpt_policy_type_t=%d, tls_policy_strings=%s..., "
+		 "tls_policy_domain=%s, mx_host_patterns=%s...)",
+		 myname, tls_policy_type,
+		 PSTR_OR_NULL(tls_policy_strings),
+		 STR_OR_NULL(tls_policy_domain),
+		 PSTR_OR_NULL(mx_host_patterns));
+
     trw->tls_policy_type = tls_policy_type;
     MYFREE_IF_SET_AND_COPY(trw->tls_policy_domain, tls_policy_domain);
     if (tls_policy_type == TLSRPT_NO_POLICY_FOUND) {
@@ -372,6 +409,11 @@ void    trw_set_tcp_connection(TLSRPT_WRAPPER *trw,
 {
     const char myname[] = "trw_set_tcp_connection";
 
+    if (msg_verbose > 1 && (snd_mta_addr || rcv_mta_name || rcv_mta_addr))
+	msg_info("%s(snd_mta_addr=%s, rcv_mta_name=%s, rcv_mta_addr=%s)",
+		 myname, STR_OR_NULL(snd_mta_addr),
+		 STR_OR_NULL(rcv_mta_name), STR_OR_NULL(rcv_mta_addr));
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -392,6 +434,9 @@ void    trw_set_ehlo_resp(TLSRPT_WRAPPER *trw, const char *rcv_mta_ehlo)
 {
     const char myname[] = "trw_set_ehlo_resp";
 
+    if (msg_verbose > 1 && rcv_mta_ehlo)
+	msg_info("%s(rcv_mta_ehlo=%s)", myname, rcv_mta_ehlo);
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -448,18 +493,18 @@ static int trw_munge_report_result(int libtlsrpt_errorcode)
 static const char *trw_failure_type_to_string(tlsrpt_failure_t failure_type)
 {
     static const NAME_CODE failure_types[] = {
-	"TLSRPT_STARTTLS_NOT_SUPPORTED", TLSRPT_STARTTLS_NOT_SUPPORTED,
-	"TLSRPT_CERTIFICATE_HOST_MISMATCH", TLSRPT_CERTIFICATE_HOST_MISMATCH,
-	"TLSRPT_CERTIFICATE_NOT_TRUSTED", TLSRPT_CERTIFICATE_NOT_TRUSTED,
-	"TLSRPT_CERTIFICATE_EXPIRED", TLSRPT_CERTIFICATE_EXPIRED,
-	"TLSRPT_VALIDATION_FAILURE", TLSRPT_VALIDATION_FAILURE,
-	"TLSRPT_STS_POLICY_FETCH_ERROR", TLSRPT_STS_POLICY_FETCH_ERROR,
-	"TLSRPT_STS_POLICY_INVALID", TLSRPT_STS_POLICY_INVALID,
-	"TLSRPT_STS_WEBPKI_INVALID", TLSRPT_STS_WEBPKI_INVALID,
-	"TLSRPT_TLSA_INVALID", TLSRPT_TLSA_INVALID,
-	"TLSRPT_DNSSEC_INVALID", TLSRPT_DNSSEC_INVALID,
-	"TLSRPT_DANE_REQUIRED", TLSRPT_DANE_REQUIRED,
-	"TLSRPT_UNFINISHED_POLICY", TLSRPT_UNFINISHED_POLICY,
+	"starttls_not_supported", TLSRPT_STARTTLS_NOT_SUPPORTED,
+	"certificate_host_mismatch", TLSRPT_CERTIFICATE_HOST_MISMATCH,
+	"certificate_not_trusted", TLSRPT_CERTIFICATE_NOT_TRUSTED,
+	"certificate_expired", TLSRPT_CERTIFICATE_EXPIRED,
+	"validation_failure", TLSRPT_VALIDATION_FAILURE,
+	"sts_policy_fetch_error", TLSRPT_STS_POLICY_FETCH_ERROR,
+	"sts_policy_invalid", TLSRPT_STS_POLICY_INVALID,
+	"sts_webpki_invalid", TLSRPT_STS_WEBPKI_INVALID,
+	"tlsa_invalid", TLSRPT_TLSA_INVALID,
+	"dnssec_invalid", TLSRPT_DNSSEC_INVALID,
+	"dane_required", TLSRPT_DANE_REQUIRED,
+	"unfinished_policY", TLSRPT_UNFINISHED_POLICY,
 	0, -1
     };
     const char *cp;
@@ -486,6 +531,11 @@ int     trw_report_failure(TLSRPT_WRAPPER *trw,
     struct tlsrpt_connection_t *con;
     int     res;
 
+    if (msg_verbose > 1)
+	msg_info("%s(failure_type=%d, additional_info=%s, failure_reason=%s)",
+		 myname, failure_type, STR_OR_NULL(additional_info),
+		 STR_OR_NULL(failure_reason));
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -510,12 +560,13 @@ int     trw_report_failure(TLSRPT_WRAPPER *trw,
     }
     trw->flags |= TRW_FLAG_REPORTED;
 
-    if (msg_verbose)
-	msg_info("%s: rpt_policy_domain=%s receiving_mx=%s failure_type=%s"
-		 " failure_reason=%s",
-		 myname, trw->rpt_policy_domain, trw->rcv_mta_name,
-		 trw_failure_type_to_string(failure_type),
-		 failure_reason ? failure_reason : "(null)");
+    /* Give the local admin a clue. */
+    msg_info("TLSRPT: status=failure, domain=%s, receiving_mx=%s[%s],"
+	     " failure_type=%s%s%s",
+	     trw->rpt_policy_domain, trw->rcv_mta_name, trw->rcv_mta_addr,
+	     trw_failure_type_to_string(failure_type),
+	     failure_reason ? ", failure_reason=" : "",
+	     failure_reason ? failure_reason : "");
 
     if ((res = tlsrpt_open(&con, trw->rpt_socket_name)) == 0) {
 	struct tlsrpt_dr_t *dr;
@@ -565,6 +616,9 @@ int     trw_report_success(TLSRPT_WRAPPER *trw)
     struct tlsrpt_connection_t *con;
     int     res;
 
+    if (msg_verbose > 1)
+	msg_info("trw_report_success");
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -579,9 +633,9 @@ int     trw_report_success(TLSRPT_WRAPPER *trw)
     }
     trw->flags |= TRW_FLAG_REPORTED;
 
-    if (msg_verbose)
-	msg_info("%s: rpt_policy_domain=%s receiving_mx=%s",
-		 myname, trw->rpt_policy_domain, trw->rcv_mta_name);
+    /* Give the local admin a clue. */
+    msg_info("TLSRPT: status=success, domain=%s, receiving_mx=%s[%s]",
+	     trw->rpt_policy_domain, trw->rcv_mta_name, trw->rcv_mta_addr);
 
     if ((res = tlsrpt_open(&con, trw->rpt_socket_name)) == 0) {
 	struct tlsrpt_dr_t *dr;
@@ -622,6 +676,13 @@ int     trw_is_reported(const TLSRPT_WRAPPER *trw)
     return (trw->flags & TRW_FLAG_REPORTED);
 }
 
+/* trw_is_skip_reused_hs - don't report TLS handshakes that reuse a session */
+
+int     trw_is_skip_reused_hs(const TLSRPT_WRAPPER *trw)
+{
+    return (trw->skip_reused_hs);
+}
+
 #endif					/* USE_TLS_RPT */
 
  /*
